﻿//-----------------------------------------------------------------------------
// <copyright file="Form1.cs" company="Eclectrics, LLC">
//     © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
// This code is from the Microsoft WCF sample that can be found here:
// http://msdn.microsoft.com/en-us/library/ms751486(VS.85).aspx
//-----------------------------------------------------------------------------

namespace NdfdTest
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.Globalization;
    using System.Text;
    using System.Windows.Forms;
    using System.Xml;

    using NdfdTest.NdfdServiceReference;
    using NdfdTest.Properties;

    /// <summary>
    /// The main application window.
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// The length of a basic zip code.
        /// </summary>
        private const int StdZipLength = 5;

        /// <summary>
        /// The NDFD web service proxy.
        /// </summary>
        private ndfdXMLPortTypeClient proxy;

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Gets a value indicating whether the contents of _zipTextBox could be a zip code.
        /// </summary>
        private bool IsZipTextValid
        {
            get
            {
                uint zipCode;

                // If the length is right, and it parses into a UINT it could be a valid zip code.
                return _zipTextBox.Text.Length == StdZipLength && uint.TryParse(_zipTextBox.Text, out zipCode);
            }
        }

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Forms.Form.Load"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.proxy = new ndfdXMLPortTypeClient("ndfdXMLPort");
        }

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Forms.Form.Closing"/> event.
        /// </summary>
        /// <param name="e">A <see cref="T:System.ComponentModel.CancelEventArgs"/> that contains the event data.</param>
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            if (this.proxy != null)
            {
                this.proxy.Close();
            }

            base.OnClosing(e);
        }

        /// <summary>
        /// Check for a valid Zip code string - and if it is valid, get the longitude/latitude.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void TextBox1TextChanged(object sender, EventArgs e)
        {
            if (this.IsZipTextValid)
            {
                PointF latLon = this.GetLatLonFromZip(_zipTextBox.Text);

                if (latLon != Point.Empty)
                {
                    _longitudeTextBox.Text = latLon.X.ToString(CultureInfo.InvariantCulture);
                    _latitudeTextBox.Text = latLon.Y.ToString(CultureInfo.InvariantCulture);
                    _forecastButton.Enabled = true;
                }
            }
            else
            {
                _longitudeTextBox.Text = string.Empty;
                _latitudeTextBox.Text = string.Empty;
                _forecastButton.Enabled = false;
            }
        }

        /// <summary>
        /// Gets the longitude and latitude assigned to the supplied 5-digit Zip code
        /// using the NDFD web service.
        /// </summary>
        /// <param name="zipString">A zip code as a string.</param>
        /// <returns>The longitude/latitude for the supplied zip code.</returns>
        private PointF GetLatLonFromZip(string zipString)
        {
            if (zipString == null)
            {
                throw new ArgumentNullException("zipString");
            }

            PointF lonLat = PointF.Empty;

            this.Cursor = Cursors.WaitCursor;

            if (this.proxy != null)
            {
                var doc = new XmlDocument();

                try
                {
                    doc.LoadXml(this.proxy.LatLonListZipCode(zipString));

                    XmlElement root = doc.DocumentElement;
                    if (root != null)
                    {
                        XmlNode latLonNode = root.SelectSingleNode("latLonList");

                        var separators = new[] { ',' };

                        if (latLonNode != null)
                        {
                            string temp = latLonNode.InnerText;
                            string[] latAndLon = temp.Split(separators, StringSplitOptions.None);

                            float latitude;
                            if (float.TryParse(latAndLon[0], out latitude))
                            {
                                lonLat.Y = latitude;
                            }

                            float longitude;
                            if (float.TryParse(latAndLon[1], out longitude))
                            {
                                lonLat.X = longitude;
                            }
                        }
                    }
                }
                catch (System.ServiceModel.CommunicationException)
                {
                    MessageBox.Show(
                        this, 
                        Resources.Invalid_Zip_Code_, 
                        Resources.Web_Service_Error, 
                        MessageBoxButtons.OK, 
                        MessageBoxIcon.Error);

                    _zipTextBox.Text = string.Empty;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            return lonLat;
        }

        /// <summary>
        /// When the user clicks the Forecast button
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs containing the event data.</param>
        private void ForecastButtonClick(object sender, EventArgs e)
        {
            // If we seem to have a longitude and latitude and a proxy...
            if (this.proxy != null && _longitudeTextBox.Text.Length > 0 && _latitudeTextBox.Text.Length > 0)
            {
                decimal longitude, latitude;

                // If the contents of the longitude and latitude text boxes parse correctly into decimals,
                // try to get the forecast.
                if (decimal.TryParse(_longitudeTextBox.Text, out longitude) && decimal.TryParse(_latitudeTextBox.Text, out latitude))
                {
                    _resultsTextBox.Text = this.GetForecastFromLatLon(longitude, latitude, _daysNumericUpDown.Text);
                }
            }
        }

        /// <summary>
        /// Get the weather forecast from the NWS NDFD.
        /// </summary>
        /// <param name="longitude">longitude for forecast</param>
        /// <param name="latitude">latitude for forecast</param>
        /// <param name="days">number of days in forecast</param>
        /// <returns>The forecast as a string.</returns>
        private string GetForecastFromLatLon(decimal longitude, decimal latitude, string days)
        {
            // This might take a while if things (Internet connection, Internet, NDFD web service) are busy...
            this.Cursor = Cursors.WaitCursor;

            string result;

            try
            {
                result = this.proxy.NDFDgenByDay(
                    latitude,
                    longitude,
                    DateTime.Now,
                    days,
                    unitType.e, 
                    formatType.Item12hourly);

                Debug.WriteLine(result);
            }
            catch (System.ServiceModel.CommunicationException comEx)
            {
                var text = new StringBuilder();
                text.AppendLine("Web Service Error: ");
                text.AppendLine(comEx.InnerException.Message);
                result = text.ToString();
            }
            catch (Exception ex)
            {
                var text = new StringBuilder();
                text.AppendLine("Error: ");
                text.AppendLine(ex.Message);
                result = text.ToString();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            return result;
        }
    }
}
