﻿// This class uses code described here:
// ms-help://MS.VSCC.v90/MS.MSDNQTR.v90.en/fxref_system.servicemodel/html/06dc0c0b-6b6b-30b8-bedd-6e965b8cac6f.htm

namespace StmEncoderLibrary
{
    using System;
    using System.ServiceModel.Channels;
    using System.ServiceModel.Description;
    using System.Xml;

    /// <summary>
    /// A binding element which specifies the message version for ISO-8859-1 encodings.
    /// </summary>
    internal class StmEncoderBindingElement : MessageEncodingBindingElement, IWsdlExportExtension
    {
        /// <summary>
        /// The reader quotas.
        /// </summary>
        private readonly XmlDictionaryReaderQuotas readerQuotas;

        /// <summary>
        /// The message version.
        /// </summary>
        private MessageVersion messageVersion;

        /// <summary>
        /// The media type.
        /// </summary>
        private string mediaType;

        /// <summary>
        /// The encoding.
        /// </summary>
        private string encoding;

        /// <summary>
        /// Initializes a new instance of the <see cref="StmEncoderBindingElement"/> class.
        /// </summary>
        /// <param name="encoding">The encoding.</param>
        /// <param name="mediaType">Type of the media.</param>
        /// <param name="msgVersion">The MSG version.</param>
        public StmEncoderBindingElement(string encoding, string mediaType, MessageVersion msgVersion)
        {
            if (encoding == null)
            {
                throw new ArgumentNullException("encoding");
            }

            if (mediaType == null)
            {
                throw new ArgumentNullException("mediaType");
            }

            if (msgVersion == null)
            {
                throw new ArgumentNullException("msgVersion");
            }

            this.messageVersion = msgVersion;
            this.mediaType = mediaType;
            this.encoding = encoding;
            this.readerQuotas = new XmlDictionaryReaderQuotas();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StmEncoderBindingElement"/> class.
        /// </summary>
        /// <param name="encoding">The encoding.</param>
        /// <param name="mediaType">Type of the media.</param>
        public StmEncoderBindingElement(string encoding, string mediaType) 
            : this(encoding, mediaType, MessageVersion.Soap11WSAddressing10)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StmEncoderBindingElement"/> class.
        /// </summary>
        /// <param name="encoding">The encoding.</param>
        public StmEncoderBindingElement(string encoding)
            : this(encoding, "text.xml")
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StmEncoderBindingElement"/> class.
        /// </summary>
        public StmEncoderBindingElement()
            : this("UTF-8")
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StmEncoderBindingElement"/> class. 
        /// Prevents a default instance of the <see cref="StmEncoderBindingElement"/> class 
        /// from being created.
        /// </summary>
        /// <param name="binding">The binding.</param>
        private StmEncoderBindingElement(StmEncoderBindingElement binding)
            : this(binding.Encoding, binding.MediaType, binding.MessageVersion)
        {
            this.readerQuotas = new XmlDictionaryReaderQuotas();
            binding.ReaderQuotas.CopyTo(this.readerQuotas);
        }

        /// <summary>
        /// When overridden in a derived class, gets or sets the message version that can be handled by the message encoders produced by the message encoder factory.
        /// </summary>
        /// <returns>The <see cref="T:System.ServiceModel.Channels.MessageVersion"/> used by the encoders produced by the message encoder factory.</returns>
        public override MessageVersion MessageVersion
        {
            get
            {
                return this.messageVersion;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                
                this.messageVersion = value;
            }
        }

        /// <summary>
        /// Gets or sets the type of the media.
        /// </summary>
        /// <value>
        /// The type of the media.
        /// </value>
        public string MediaType
        {
            get
            {
                return this.mediaType;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }

                this.mediaType = value;
            }
        }

        /// <summary>
        /// Gets or sets the encoding.
        /// </summary>
        /// <value>
        /// The encoding.
        /// </value>
        public string Encoding
        {
            get
            {
                return this.encoding;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }

                this.encoding = value;
            }
        }

        /// <summary>
        /// Gets the reader quotas. This encoder does not enforces any quotas for 
        /// unsecured messages. The quotas are enforced for the secure portions of 
        /// messages when this encoder is used in a binding that is configured with 
        /// security.
        /// </summary>
        public XmlDictionaryReaderQuotas ReaderQuotas
        {
            get { return this.readerQuotas; }
        }

        #region IMessageEncodingBindingElement Members

        /// <summary>
        /// When overridden in a derived class, creates a factory for producing message encoders.
        /// </summary>
        /// <returns>
        /// The <see cref="T:System.ServiceModel.Channels.MessageEncoderFactory"/> used to produce message encoders.
        /// </returns>
        public override MessageEncoderFactory CreateMessageEncoderFactory()
        {
            return new StmEncoderFactory(this.MediaType, this.Encoding, this.MessageVersion);
        }

        #endregion

        /// <summary>
        /// When overridden in a derived class, returns a copy of the binding element object.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.ServiceModel.Channels.BindingElement"/> object that is a deep clone of the original.
        /// </returns>
        public override BindingElement Clone()
        {
            return new StmEncoderBindingElement(this);
        }

        /// <summary>
        /// Builds the channel factory.
        /// </summary>
        /// <typeparam name="TChannel">The type of the channel.</typeparam>
        /// <param name="context">The context.</param>
        /// <returns>The channel factory.</returns>
        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.BuildInnerChannelFactory<TChannel>();
        }

        /// <summary>
        /// Determines whether this instance [can build channel factory] the specified context.
        /// </summary>
        /// <typeparam name="TChannel">The type of the channel.</typeparam>
        /// <param name="context">The context.</param>
        /// <returns>
        ///   <c>true</c> if this instance [can build channel factory] the specified context; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            return context.CanBuildInnerChannelFactory<TChannel>();
        }

        /// <summary>
        /// Builds the channel listener.
        /// </summary>
        /// <typeparam name="TChannel">The type of the channel.</typeparam>
        /// <param name="context">The context.</param>
        /// <returns>The channel listener.</returns>
        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.BuildInnerChannelListener<TChannel>();
        }

        /// <summary>
        /// Determines whether this instance [can build channel listener] the specified context.
        /// </summary>
        /// <typeparam name="TChannel">The type of the channel.</typeparam>
        /// <param name="context">The context.</param>
        /// <returns>
        ///   <c>true</c> if this instance [can build channel listener] the specified context; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanBuildChannelListener<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.CanBuildInnerChannelListener<TChannel>();
        }

        /// <summary>
        /// Gets the property.
        /// </summary>
        /// <typeparam name="T">The type of the class.</typeparam>
        /// <param name="context">The context.</param>
        /// <returns>The requested property.</returns>
        public override T GetProperty<T>(BindingContext context)
        {
            if (typeof(T) == typeof(XmlDictionaryReaderQuotas))
            {
                return (T)(object)this.readerQuotas;
            }

            return base.GetProperty<T>(context);
        }

        #region IWsdlExportExtension Members

        /// <summary>
        /// Writes custom Web Services Description Language (WSDL) elements into the generated WSDL for a contract.
        /// </summary>
        /// <param name="exporter">The <see cref="T:System.ServiceModel.Description.WsdlExporter"/> that exports the contract information.</param>
        /// <param name="context">Provides mappings from exported WSDL elements to the contract description.</param>
        void IWsdlExportExtension.ExportContract(WsdlExporter exporter, WsdlContractConversionContext context)
        {
        }

        /// <summary>
        /// Writes custom Web Services Description Language (WSDL) elements into the generated WSDL for an endpoint.
        /// </summary>
        /// <param name="exporter">The <see cref="T:System.ServiceModel.Description.WsdlExporter"/> that exports the endpoint information.</param>
        /// <param name="context">Provides mappings from exported WSDL elements to the endpoint description.</param>
        void IWsdlExportExtension.ExportEndpoint(WsdlExporter exporter, WsdlEndpointConversionContext context)
        {
            // The MessageEncodingBindingElement is responsible for ensuring that the WSDL has the correct
            // SOAP version. We can delegate to the WCF implementation of TextMessageEncodingBindingElement for this.
            var mebe = new TextMessageEncodingBindingElement { MessageVersion = this.messageVersion };
            ((IWsdlExportExtension)mebe).ExportEndpoint(exporter, context);
        }

        #endregion IWsdlExportExtension Members
    }
}
