﻿// This class uses code described here:
// ms-help://MS.VSCC.v90/MS.MSDNQTR.v90.en/fxref_system.servicemodel/html/a447d456-8eb9-58dd-8f5d-3430a58998c1.htm

namespace StmEncoderLibrary
{
    using System.ServiceModel.Channels;

    /// <summary>
    /// A factory for creating Iso88501 encoders.
    /// </summary>
    public class StmEncoderFactory : MessageEncoderFactory
    {
        /// <summary>
        /// The encoder produced by the factory.
        /// </summary>
        private readonly MessageEncoder encoder;

        /// <summary>
        /// The message version used by the encoder.
        /// </summary>
        private readonly MessageVersion version;

        /// <summary>
        /// The media type used by the encoder
        /// </summary>
        private readonly string mediaType;

        /// <summary>
        /// The charset used by the encoder
        /// </summary>
        private readonly string charSet;

        /// <summary>
        /// Initializes a new instance of the StmEncoderFactory class. The factory knows
        /// how to create StmEncoders that can read and write to a stream for various 
        /// types of message encoding
        /// </summary>
        /// <param name="mediaType">The media type used by the encoder.</param>
        /// <param name="charSet">The charset used by the encoder.</param>
        /// <param name="version">The message version used by the encoder.</param>
        internal StmEncoderFactory(string mediaType, string charSet, MessageVersion version)
        {
            this.mediaType = mediaType;
            this.charSet = charSet;
            this.version = version;
            this.encoder = new StmEncoder(this);
        }

        /// <summary>
        /// When overridden in a derived class, gets the message encoder that is produced by the factory.
        /// </summary>
        /// <returns>The <see cref="T:System.ServiceModel.Channels.MessageEncoder"/> used by the factory.</returns>
        public override MessageEncoder Encoder
        {
            get { return this.encoder; }
        }

        /// <summary>
        /// When overridden in a derived class, gets the message version that is used by the encoders produced by the factory to encode messages.
        /// </summary>
        /// <returns>The <see cref="T:System.ServiceModel.Channels.MessageVersion"/> used by the factory.</returns>
        public override MessageVersion MessageVersion
        {
            get { return this.version; }
        }

        /// <summary>
        /// Gets the type of the media.
        /// </summary>
        /// <value>
        /// The type of the media.
        /// </value>
        public string MediaType
        {
            get { return this.mediaType; }
        }

        /// <summary>
        /// Gets the char set.
        /// </summary>
        public string CharSet
        {
            get { return this.charSet; }
        }
    }
}
